import os
import pickle

from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.linear_model import LogisticRegression


class Sentimental:

    def __init__(self,
                 max_ngrams=1,
                 min_df=0.0,
                 max_df=1.0,
                 max_features=None):
        self.n_grams = max_ngrams
        self.max_df = max_df
        self.min_df = min_df
        self.max_features = max_features
        self.labels = ['negative', 'positive']

    def train(self, corpus_folders):
        self.vectorizer = TfidfVectorizer(analyzer='word',
                                          ngram_range=(1, self.n_grams),
                                          max_df=self.max_df,
                                          min_df=self.min_df,
                                          max_features=self.max_features,
                                          decode_error='replace')

        x_input = []
        y_target = []
        for folder in corpus_folders:
            for i in range(len(self.labels)):
                label = self.labels[i]
                data_file = '%s/%s_examples.txt' % (folder, label)
                with open(data_file, 'r') as f:
                    read_data = f.readlines()
                    x_input.extend(read_data)
                    y_target.extend([i] * len(read_data))

        x_train = self.vectorizer.fit_transform(x_input)

        self.predictor = LogisticRegression()
        self.predictor.fit(x_train, y_target)

    def sentiment(self, text):
        x = self.vectorizer.transform([text])
        probabilities = self.predictor.predict_proba(x)[0]
        res = {}
        for i in range(len(self.labels)):
            res[self.labels[i]] = probabilities[i]
        return res

    def save(self, output_file):
        with open(output_file, 'wb') as f:
            pickle.dump(self, f)

    def load(pickel_file):
        with open(pickel_file, 'rb') as f:
            return pickle.load(f)

    def get_datafolder():
        return os.path.dirname(__file__) + '/data'



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of sentimental.
# https://github.com/ErikGartner/sentimental

# Licensed under the - license:
# http://www.opensource.org/licenses/--license
# Copyright (c) 2016, Erik Gärtner <erik@gartner.io>

from sentimental.version import __version__  # NOQA
from sentimental.sentimental import Sentimental
from sentimental.example_extractor import ExampleExtractor



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of sentimental.
# https://github.com/ErikGartner/sentimental

# Licensed under the - license:
# http://www.opensource.org/licenses/--license
# Copyright (c) 2016, Erik Gärtner <erik@gartner.io>

from setuptools import setup, find_packages
from sentimental import __version__

tests_require = [
    'mock',
    'nose',
    'coverage',
    'yanc',
    'preggy',
    'tox',
    'ipdb',
    'coveralls',
    'sphinx',
]

setup(
    name='sentimental',
    version=__version__,
    description='A sentiment analyzer',
    long_description=
    '''Sentiment analysis made easy; built on top off solid libraries.

Sentimental uses Scikit-learn to perform easy sentiment analysis. The idea is to create a simple out-of-box solution that yields acceptable results without complex configuration. Sentimental also uses a simple format for its training corpora that makes it easy to add more training data.''',
    keywords=['sentiment analysis', 'sentiment', 'nlp'],
    author='Erik Gärtner',
    author_email='erik@gartner.io',
    url='https://github.com/ErikGartner/sentimental',
    download_url='https://github.com/ErikGartner/sentimental/archive/' +
    __version__ + '.tar.gz',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Operating System :: Unix',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Operating System :: OS Independent',
    ],
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'numpy>=1.11.0',
        'scipy>=0.17.0',
        'scikit-learn>=0.17.1',
        'pyahocorasick>=1.1.0',
    ],
    extras_require={
        'tests': tests_require,
    },
    entry_points={
        'console_scripts': [
            # add cli scripts here in this form:
            # 'sentimental=sentimental.cli:main',
        ],
    }, )



import os
import re
from collections import Counter

import ahocorasick


class ExampleExtractor:
    def __init__(self):
        self.indicators = {}

    def load_labeled_words(self, word_file, label):
        word_set = self.indicators.get(label, set())
        with open(word_file, 'r') as f:
            word_set.update([x.lower().strip() for x in f.readlines()])
        self.indicators[label] = word_set

    def extract_examples(self, corpus_file, output_dir, cutoff=2):
        with open(corpus_file, 'r') as f:
            text = f.read()
        text = re.sub(r'\s', ' ', text)
        sentences = re.split(r'(?<!\w\.\w.)(?<![A-Z][a-z]\.)(?<=\.|\?)\s',
                             text)

        labeled = {l: [] for l in self.indicators}

        automaton = ahocorasick.Automaton()
        for label in self.indicators:
            for indi in self.indicators[label]:
                automaton.add_word(indi, label)
        automaton.make_automaton()

        for sentence in sentences:
            sentence = sentence.strip()
            c = Counter([label for (pos, label) in automaton.iter(sentence)])

            common = c.most_common(2)
            if len(common) >= 2 and common[0][1] - common[1][1] >= cutoff:
                labeled[common[0][0]].append(sentence)
            elif len(common) == 1 and common[0][1] >= cutoff:
                labeled[common[0][0]].append(sentence)

        for label in labeled:
            with open('%s/%s_examples.txt' % (output_dir, label), 'w') as f:
                f.write('\n'.join(labeled[label]))
